typedef struct
{
	unsigned long nver;    // version of the SDB file. Use 101
   unsigned long ntruss;  // number of truss elements
   unsigned long nbeam;   // number of beam elements
	unsigned long nmembr;  // number of membrane elements
	unsigned long nplate;  // number of plate elements
	unsigned long nsolid;  // number of solid elements
   unsigned long nspring; // number of spring elements
   unsigned long nlcase;  // number of load cases
} Outheads;

// File records
/*-----------------------------------------------------------------
Outheads                  ---> output header

nlcase * u long           ----> truss_loc (offset of truss stresses for each load case)
nlcase * double         ----> maxtruss    (maximum axial force in module for each load case)

nlcase * u long           ----> beam_loc  (offset of beam stresses for each load case)
nlcase * 6 * double     ----> maxbeams    (maximum 6 internal stresses in module for each load case)

nlcase * u long           ----> spring_loc (offset of spring stresses for each load case)
nlcase * 2 * double     ----> maxspring    (maximum 2 internal stresses in module for each load case)

nlcase * u long           ----> membr_loc  (offset of membrane stresses for each load case)
nlcase * 8 * double     ----> maxmembr  sI, sII, sIII, sVM, free, free, free, free (maxima for each load case)

nlcase * u long           ----> plate_loc  (offset of plate element stresses for each load case)
nlcase * 6 * double     ----> maxplate  sI, sII,sIII, sVM, free, free   (maxima for each load case)

nlcase * u long           ----> solid_loc  (offset of solid element stresses for each load case)
nlcase * 6 * double     ----> maxsolid  sI, sII,sIII, sVM, free, free (maxima for each load case)

i= 1 to nlcase
		j = 1 to ntruss
		       double              // axial force. Positive if traction; negative if compression
		endj
endi

i= 1 to nlcase
		j = 1 to nbeam
		       12 * double        // 6 forces at first extreme; 6 forces at second extreme
		endj
endi

i= 1 to nlcase
		j = 1 to nspring
		       2 * double         // 2 forces  if translational spring the first is the right one
				                    //           if rotational spring then the second is the right one
										  //           always two numbers are written. They are the force (or moment)
										  //           inside the spring. 
		endj
endi


i= 1 to nlcase
		j = 1 to nmembr
	       32 double = 4 * (s1, s2, s3, t12, sI, sII, sIII, sVM) double  (s1, s2, s3, t12 in local ref)
		endj
endi


i= 1 to nlcase
		j = 1 to nplate
	       32 double = 4 * (s1, s2, t12, m1, m2, m12, q1, q2) double  (all in local ref)
		endj
endi


i= 1 to nlcase
		j = 1 to nsolid
	       80 double = 8 * (s1, s2, s3, t12, t23, t31, sI, sII, sIII, sVM) double
		endj
endi

----------------------------
EXPLANATIONS
---------------------------

All data must be in: N, mm, and derived units as applicable

Truss force: is in Newton
Beam forces: are in Newton
Beam moments: are in Nmm
Membrane stresses: are in N/mm^2

Beam forces are 6+6. Each 6 are:
(F1, F2, F3, M1, M2, M3) at first and second extreme
At first extreme the local tern axis 1 "enters" into the element; at second extreme the
local tern axis 1 "exits" from the element. Local tern is always the same, at first and second
extreme.

For instance a constant traction of 20 would be described by:
F1 = -20   at first extreme
F1 =  20   at second extreme

a constant compression by:
F1 =  20   at first extreme
F1 = -20   at second extreme

Special symbols:

sI         first principal stress
sII        second principal stress
sIII       third principal stress
sVM        Von Mises stress
m1         bending per unit length (plate) caused by sx
m2         bending per unit length (plate) caused by sy
m12        bending per unit length
q1         shear per unit length (only computed for thick plates)
q2         shear per unit length (only computed for thick plates)
s1         normal stress
s2         normal stress
s3         normal stress
t12        shear stress
t23        shear stress
t31        shear stress


Stresses are output at node locations in local reference system.


Here is an example of location computation

ntruss = 103
nbeam = 97
nmembr = 23
nplate = 0
nsolid = 0
nspring = 0
nlcase = 11

Location of membr stresses for case 5=

sizeof(Outheads) + 
11 * (6 * sizeof(unsigned long) + 29 * sizeof(double)) +
11 * 103 * sizeof(double) +
11 * 97 * 12 * sizeof(double) +
4  * 23 * 32 * sizeof(double)

that is :
sizeof(Outheads) + 
nlcase * (6 * sizeof(unsigned long) + 29 * sizeof(double)) +
nlcase * ntruss * sizeof(double) +
nlcase * nbeam * 12 * sizeof(double) +
(nlcase -1)  * nmembr * 32 * sizeof(double)


Locations are needed for quick data access and they must be properly written into the SDB file.

*/