#ifndef SR3_H
#define SR3_H

/*

	File version 2.0:

	  Modified SR3_Combi_Header structure
	  Modified the data block referring to Combinations ("item" block added, to manage combinations
	      of combinations). For each Combi two new vectors are defined: a) A vector of multipliers (double);
			b) A vector of combination number. Both vectors have size "nitem", defined in Combi Header.
			If "nitem" is 0 no vectors are added.
	  Cfload structure: qualifier "qual" must be 1 or 2 not 5 or 6 as defined in file version 1.0.

*/

#define SR3_ATTRNUM 5
#define SR3_NAMLEN 20
#define SR3_PROPLEN 20
#define SR3_MATNAMLEN 20

#define SR3_NUMRIGHTCOLS 10           /* columns in the menu region */
#define SR3_LABELLEN 20
#define SR3_LCIDENTSIZE  60	// Windows


// char:          1 byte
// unsigned char: 1 byte
// long:          4 bytes
// unsigned long: 4 bytes
// double         8 bytes

/* general backup buffers */
typedef struct
{
   unsigned long version;     // file version : current is 200 ("2.00")
	char program[300]; // the program which has written this file
	char job[300];     // a job identifier
	char post;         // 0 post processing not available; 1 post processing available
	                   // if post = 1 -> then .sdb .ddb .rdb must also be created
} SR3_Backup_buffer;


typedef struct
{
   unsigned char assect_num; // assigned section num
   unsigned char asmat_num;  // assigned material num
   unsigned char asprop_num; // assigned properties (plate & membrane) num
   unsigned long nlcase;     // number of load cases
   unsigned long ncombi;     // number of load combinations (linear combinations)
   unsigned long node_num;   // number of fem nodes
   unsigned long truss_num;  // number of truss elements
   unsigned long beam_num;   // number of beam elements
   unsigned long plate_num;  // number of plate-shell elements (plate+membrane)
   unsigned long membr_num;  // number of pure membrane elements
   unsigned long solid_num;  // number of solid elements
   unsigned long spring_num; // number of spring elements (a spring is always connected to "ground")
	unsigned long group_num;  // number of different groups; must be at least 1
   unsigned long dummy1; // currently 0
   unsigned long dummy2; // currently 0
   unsigned long dummy3; // currently 0
   unsigned long dummy4; // currently 0
   unsigned long dummy5; // currently 0
   unsigned long dummy6; // currently 0
   double l_conv; // number of millimeters in current length unit (=1)
   double f_conv; // number of Newtons in current force unit (=1)
   double t_conv; // number of seconds in current time unit (=1)
   double temp_conv;  // = 1  if C;        = 1.8 if F; MUST be 1
   double temp_conv2; // = 0. if C;        = 32. if F; MUST be 0
} SR3_Backup_buffer1;

// main fem objects
typedef struct
{
	double nc[3];        // x y z in global ref
	char  tx;            // x translation constraint code. 1: constraint ; 0: free
	char  ty;            // y translation constraint code. 1: constraint ; 0: free
	char  tz;            // z translation constraint code. 1: constraint ; 0: free
	char  rx;            // x rotation constraint code. 1: constraint ; 0: free
	char  ry;            // y rotation constraint code. 1: constraint ; 0: free
	char  rz;            // z rotation constraint code. 1: constraint ; 0: free
   double mass[6];      // translational x,y, z, rotational x,y,z masses; e.g. mass[3]=mrx
	char name[21];       // name of this node (can be null) - must be null terminated
	unsigned long group; // the unique group number to which this node belongs <= group_number
} SR3_R3node;

typedef struct
{
   unsigned long node[2];  // first node number and second node number define axis 1 (1->2)
	unsigned char  mat;     // material number in the material array
	unsigned char  section; // cross-section number in the cross-section array
	// Release and connection codes - if exist one e1_xx=1       then e1_cc = 1 necessarily;
	//                                if all e1_xx = 0           then e1_cc = 1 optionally.
	//
	//                              - if exist one e2_xx=1       then e2_cc = 1 necessarily;
	//                                if all e2_xx = 0           then e2_cc = 1 optionally.
	char  e1_t1;  // release code first extreme; axis 1 translation; 1: released; 0: unreleased
	char  e1_t2;  // release code first extreme; axis 2 translation; 1: released; 0: unreleased
	char  e1_t3;  // release code first extreme; axis 3 translation; 1: released; 0: unreleased
	char  e1_r1;  // release code first extreme; axis 1 rotation; 1: released; 0: unreleased
	char  e1_r2;  // release code first extreme; axis 2 rotation; 1: released; 0: unreleased
	char  e1_r3;  // release code first extreme; axis 3 rotation; 1: released; 0: unreleased
	char  e1_cc;  // connection code first extreme; 1: interrupted; 0: uninterrupted
	//
	char  e2_t1;  // release code second extreme; axis 1 translation; 1: released; 0: unreleased
	char  e2_t2;  // release code second extreme; axis 2 translation; 1: released; 0: unreleased
	char  e2_t3;  // release code second extreme; axis 3 translation; 1: released; 0: unreleased
	char  e2_r1;  // release code second extreme; axis 1 rotation; 1: released; 0: unreleased
	char  e2_r2;  // release code second extreme; axis 2 rotation; 1: released; 0: unreleased
	char  e2_r3;  // release code second extreme; axis 3 rotation; 1: released; 0: unreleased
	char  e2_cc;  // connection code second extreme; 1: interrupted; 0: uninterrupted
	// End springs
   double kfi[4]; // kfi[0] : end 1 axis 2 rotational stiffness [if != 0. e1_r2 = 1 necessarily]
                  // kfi[1] : end 1 axis 3 rotational stiffness [if != 0. e1_r3 = 1 necessarily]
                  // kfi[2] : end 2 axis 2 rotational stiffness [if != 0. e2_r2 = 1 necessarily]
                  // kfi[3] : end 2 axis 3 rotational stiffness [if != 0. e2_r3 = 1 necessarily]

	char name[21]; // name of the object (can be null) - must be null terminated
	unsigned long group; // the unique group number to which this beam belongs <= group_number
	unsigned char subkind; // presently dummy
	// Offsets
	double ecc[6];
	// Orientation
	unsigned char oriecode; // can be 2 or 3; if 2 then "vect" and "axis 1" define plane 1-2
	                        //                if 3 then "vect" and "axis 1" define plane 1-3
	double vect[3];         // a not null vector not parallel to axis 1
	// Winkler constants
	double KWink[4];        // all zero for not Winkler beams; unreferenced for Winkler beams
} SR3_Beam;


typedef struct
{
   unsigned long node[2]; // first node number and second node number define axis 1 (1->2)
	unsigned char  mat;    // material number in the material array
	unsigned char  section;// cross-section number in the cross-section array
	char name[21];         // name of the object (can be null) - must be null terminated
	unsigned long group;   // the unique group number to which this truss belongs <= group_number
	unsigned char subkind; // presently dummy
	unsigned char oriecode;// can be 2 or 3; if 2 then "vect" and "axis 1" define plane 1-2
	                       //                if 3 then "vect" and "axis 1" define plane 1-3
	double vect[3];        // a not null vector not parallel to axis 1
} SR3_Truss;

typedef struct
{
   unsigned long node[9]; // not more than 4 nodes allowed
   unsigned char nnode;  // number of nodes; can be 3 or 4
   unsigned char mat;     // material number in the material array
   unsigned char propr;   // property number in the property array
	char name[21];         // name of the object (can be null) - must be null terminated
	unsigned long group;   // the unique group number to which this plate-shell belongs <= group_number
	unsigned char subkind; // 0 thin plate; 1 thick plate.
	double KWink;          // Winkler constant
} SR3_Plate;

typedef struct
{
   unsigned long node[9]; // not more than 4 nodes allowed
   unsigned char nnode;  // number of nodes; can be 3 or 4
   unsigned char mat;     // material number in the material array
   unsigned char propr;   // property number in the property array
	char name[21];         // name of the object (can be null) - must be null terminated
	unsigned long group;   // the unique group number to which this membrane belongs <= group_number
	unsigned char subkind; // currently dummy
} SR3_Membr;

typedef struct
{
   unsigned long node[20]; // not more than 8 nodes (brick) allowed
   unsigned char nnode;   // it is: 4 (tetrahedron); 6 (pentahedron); 8 (hexahedron)
   unsigned char mat;      // material number in the material array
	char name[21];          // name of the object (can be null) - must be null terminated
	unsigned long group;    // the unique group number to which this solid belongs <= group_number
	unsigned char subkind;  // presently dummy
} SR3_Solid;

typedef struct
{
	unsigned long node;  // the node of the spring or settle (assigned displacement)
	double dir[3];       // its direction vector in global cs
	double value;        // stiffness value (if settle a very high stiffness must be input)
	double svalue;       // settle value (if 0 it is a simple spring)
	unsigned char rotation; // 1 if rotational; 0 if translational
	unsigned long lca;      // load case number only used if svalue != 0 (not null)
	char name[21];          // name of the object (can be null) - must be null terminated
	unsigned long group;    // the unique group number to which this solid belongs <= group_number
	unsigned char subkind;  // presently dummy
} SR3_Spring;


// other data struct
typedef struct
{
   char name[40];       // group name (identifier) - must be null terminated
   unsigned long color; // COLORREF assigned to this group
} SR3_Group;

// load cases

/* load case header */
typedef struct
{
   char ident[SR3_LCIDENTSIZE]; // load case identifier (a name) - must be null terminated
   unsigned char kind;          // load case kind, one of the following
	//#define GRAVITY 0          gravity loads
	//#define DEAD 1             permanent loads
	//#define LIVE 2             live loads
	//#define SNOW 3             snow loads
	//#define WIND 4             wind loads
	//#define EARTH 5            earthquake equivalent static analysis load
	//#define THERM 6            thermal loads
	//#define EARTHMD 7          earthquake modal + response spectrum analysis loads

	// number of actions in this load case (can be 0)
	unsigned long gforces_num; // General forces number (applied to nodes)
	unsigned long cfloads_num; // Concentrated forces/couples number (applied to beams)
	unsigned long uloads_num;  // Uniform loads number (applied to beams)
	unsigned long lloads_num;  // Linear loads number (applied to beams)
	unsigned long glloads_num; // General linear loads number (applied to beams)
	unsigned long tloads_num;  // Temperature loads number (applied to trusses, beams, plates,
	                           // membranes, and solids)
} SR3_Lc_header;

/* combi header */
// modified in file version 2: added selected field and nitem field
typedef struct
{
   char ident[SR3_LCIDENTSIZE]; // combi identifier (a name) - must be null terminated
	                             // membranes, and solids)
	char kind;    // 0 linear additive; 
		           // 1 sum max abs(+); 
		           // 2 sum max abs(-);
		           // 3 SRSS (+)
		           // 4 SRSS (-)
		           // 5 Range (+)
		           // 6 Range (-)
		           // 7 Envelope (max)
		           // 8 Envelope (min)
	char selected;  // 1 if the combination is active/selected, 0 otherwise
	unsigned long nitem; // the number of items of this combination, i.e. (combination, factor) pairs

} SR3_Combi_Header;

// Gforce: generalized "forces" acting on nodes
typedef struct
{
	unsigned char qual;  // can be 1 (force) or 2 (moment)
	unsigned long node;
	double val[3];       // generalized force components (x, y, z)
} SR3_Gforce;

// Uload: uniform distributed loads acting on beam elements (from end 1 to end 2)
typedef struct
{
	unsigned char qual; // can be 3 (force) or 4 (moment)
   unsigned char ref;  // reference system can be 0 (global) or 1 (local)
	unsigned long bnum; // beam number
	double val[3];      // components in proper reference system
} SR3_Uload;

// Cfload: concentrated force or moment acting on beam elements 
typedef struct
{
	unsigned char qual; // can be 1 (force) or 2 (moment)
   unsigned char ref;  // reference system can be 0 (global) or 1 (local)
	unsigned long bnum; // beam number
	double absc;        // non dimensional abscissa ranging from 0. (end 1) to 1. (end 2)
	double val[3];      // force or moment components in proper reference system
} SR3_Cfload;

// Lload; linear distributd force or moment acting on beam elements. From end 1 to end 2
typedef struct
{
	unsigned char qual; // can be 7 (force) or 8 (moment)
   unsigned char ref;  // reference system can be 0 (global) or 1 (local)
	unsigned long bnum; // beam number
	double val[2][3];   // val[0][i] first extreme; val[1][i] second extreme; i=0,1,2 (rs axis)
} SR3_Lload;

// Glload: general linear load, distributed force or moment acting on beam elements
//         from an internal point "A" to an internal point "B"
typedef struct
{
	unsigned char qual;  // can be 9 (force) or 10 (moment)
   unsigned char ref;   // reference system can be 0 (global) or 1 (local)
	unsigned long bnum;  // beam number
	double abscA;        // non dimensional abscissa point A ranging from 0. (end 1) to 1. (end 2)
	double abscB;        // non dimensional abscissa point B ranging from 0. (end 1) to 1. (end 2)
	double val[2][3];    // val[0][i] point A; val[1][i] point B; i=0,1,2 (rs axis)
} SR3_Glload;

// Tload: thermal load acting on trusses, beams, plates, membranes, or solids
typedef struct
{
	unsigned char qual; // beam: 13; truss: 14; plate: 101; membrane: 102; solid: 104
	unsigned long num;  // number of the finite element
	double val;         // temperature value
} SR3_Tload; 



// properties
// Elastic material data
typedef struct
{
   double g;  // weight per unit volume
   double E;  // Young's modulus
   double nu; // Poisson's coefficient
   double alpha; // thermal coefficient
   double fy;    // yield stress or similar
   double ft;    // ultimate stress or similar
   char attr;    // 1 Euro; 2 USA; 3 none of the previous
} SR3_Mprops;


typedef struct
{
   char label[SR3_MATNAMLEN]; // must be null terminated
   SR3_Mprops properties;
} SR3_Material;

// Cross-sections available kinds (see also shapes.doc)
// sm: smooth (i.e. rolled)
// sh: sharp  (i.e. welded)
// cf: cold formed

// cross-sections with * can be recomputed by sizes (e.g. (b,h))
// cross-sections with ^ cannot be recomputed by sizes
enum cross_section_kind
{ SR3_Hsm = '\1',   // H smooth radius (rolled I or H) *
  SR3_Ipn,          // European IPN ^
  SR3_Usm,          // European UPN *
  SR3_Lsm,          // Angle *
  SR3_Tsm,          // Tee tapered web sides ^
  SR3_Psh,          // Plate or rectangular *
  SR3_O_,           // Round or CHS *
  SR3_Osm,          // un-used
  SR3_Hsh,          // I or H welded *
  SR3_Lsh,          // L *
  SR3_Ush,          // U *
  SR3_Tsh,          // Tee *
  SR3_Osh,          // Box *
  SR3_OshG,         // un-used
  SR3_U_O,          // [] *
  SR3_U_H,          // ][ *
  SR3_Usm_O,        // un-used
  SR3_Usm_H,        // un-used
  SR3_Hsm_I,        // un-used
  SR3_HHsm,         // un-used
  SR3_Hsm2Usm,      // un-used
  SR3_L2_T_sm,      // _||_  2 Lsm  *
  SR3_L2_CR_sm,     //       2 Lsm   (cross) *
  SR3_L4_CR_sm,     //       4 Lsm   (cross) *
  SR3_Juan,         //  Totally generic ^
  SR3_Rhs,          // Rectangular hollow sections *
  SR3_Composed,     // Generic composed multi section *
  SR3_Cold,         // Generic cold-formed *
  SR3_Omcf,         // Hat (omega) cold formed *
  SR3_Ucf,          // U cold formed *
  SR3_Zcf,          // Z cold formed *
  SR3_Lcf,          // L cold formed *
  SR3_U_H_cf,       // ][ cold formed *
  SR3_Poli,         // generic polygon(s) made up section *
  SR3_Thsm};        // Hsm cut into a Tee *


// this struct is used for Cold sections
typedef struct
{
   char qual;	         // STRAIGHT = 1; CIRCULAR = 2
   char hole;	         // this side is a hole (=1), or not (=0)
   double thickness;		// thickness of the side
   double param[8];		// parameter vector
	                     // if side is straight, then:
	                     //     param[0] = x1    param[1] = y1
	                     //     param[0] = x2    param[1] = y2
								// if side is circular, then:
	                     //     param[0] = xstart            param[1] = ystart
	                     //     param[2] = xend              param[3] = yend
	                     //     param[4] = xcenter           param[5] = ycenter
	                     //     param[6] = beta              param[7] = average radius
								//     beta is the arc length / radius i.e. radians positive counter cloack wise
} SR3_sh_side;


typedef struct
{
	char code;               // can be Hsm, Hsh,... one of the kind defined previously
	char name[SR3_NAMLEN];  // cross-section name must be null terminated
} SR3_Shape;

/* a size is read according to the following rules:

1)     a SR2_Shape data struct which specifies the name and type of the cross section to be read (Hsm, etc)
2)     if the cross section is not Juan, Cold, Composed or Poli a vector of 10 doubles
       is always read; there cross-section sizes will be written (not more than 10 parameters
		 are needed)
3)     special rules are set up for each of these peculiar shapes: Juan, Cold, Composed, Poli
       see also file Shapes.doc for more info and explanations.







*/

// Bidimensional element properties
typedef struct
{
   char name[SR3_PROPLEN]; // property name - must be null terminated
   double t;               // constant thickness
   double J;               // inertia per unit length (1/12)t^3 or different
} SR3_Plmprop;

///////////////////////////////////////////////////////////////////////////////////
//
//
//
//
//                         File .SR3  structure and records
//
//
//
//
///////////////////////////////////////////////////////////////////////////////////
//
//  The fem model written into this file MUST have first six nodes reserved
//  The six nodes are used as third nodes by old program version.
//  The first six nodes must be fully constrained.
//  They must have the following coords (X, Y, Z):
//  Node 1: (0, 1.e14, 0.)
//  Node 2: (0, 0, 1.e14)
//  Node 3: (1.e14, 0, 0.)
//  Node 4: (0, -1.e14, 0.)
//  Node 5: (0, 0, -1.e14)
//  Node 6: (-1.e14, 0, 0.)
//
// The unit of measure must be the following:
//
// mm as length unit
// N  as force unit
// sec as time unit
// C as temperature unit
// a 1Kg mass is iput as  0.001
/*
   Number of items     Item

                 1     SR3_Backup_buffer   // initial struct
                 1     SR3_Backup_buffer1  // initial struct1


          node_num     SR3_R3node       // nodes


          beam_num     SR3_Beam         // beams


	   	 truss_num    SR3_Truss        // trusses


          plate_num    SR3_Plate        // plates


			 membr_num    SR3_Membr        // membranes


          solid_num    SR3_Solid        // solids


			 spring_num   SR3_Spring       // springs


          group_num    SR3_Group        // groups


			    nlcase    SR3_Lc_header    // load cases
  [case1] gforces_num    SR3_Gforce       // applied actions
  [case1] uloads_num     SR3_Uload
  [case1] cfloads_num    SR3_Cfload
  [case1] lloads_num     SR3_Lload
  [case1] glloads_num    SR3_Glload
  [case1] tloads_num     SR3_Tload
  [case2] gforces_num    SR3_Gforce
  [case2] uloads_num     SR3_Uload
  [case2] cfloads_num    SR3_Cfload
  [case2] lloads_num     SR3_Lload
  [case2] glloads_num    SR3_Glload
  [case2] tloads_num     SR3_Tload
...................................
[case nlcase] gforces_num    SR3_Gforce
[case nlcase] uloads_num     SR3_Uload
[case nlcase] cfloads_num    SR3_Cfload
[case nlcase] lloads_num     SR3_Lload
[case nlcase] glloads_num    SR3_Glload
[case nlcase] tloads_num     SR3_Tload         // end applied actions


  [combi1]  1               SR3_Combi_Header   // start combinations - combi header for combi 1
  [combi1] nlcase            double            // combinations. multiplier of load case1, case2,..,case nlcase 
  [combi1] nitem             double            // multiplier of item from file version 2.00
  [combi1] nitem             unsigned long     // item number  from file version 2.00
  [combi2]  1               SR3_Combi_Header   // combi header for combi 2
  [combi2] nlcase            double
  [combi2] nitem             double            // multiplier of item
  [combi2] nitem             unsigned long     // item number
..................................
[combi ncombi]  1           SR3_Combi_Header    // combi header for combi ncombi
[combi ncombi] nlcase        double             // load case multiplier
[combi ncombi] nitem         double             // multiplier of item from file version 2.00
[combi ncombi] nitem         unsigned long      // item number from file version 2.00
                                                // end combinations

         asmat_num           SR3_Material       // materials


			asprop_num          SR3_Plmprop        // bidim elements properties


[cross-section 1]  1         SR3_Shape          // initial cross-section data
[cross-section 1]  1         "block"            // "block" is defined below
[cross-section 2]  1         SR3_Shape          // initial cross-section data
[cross-section 2]  1         "block"            // "block" is defined below
.............
[cross-section assect_num]  1         SR3_Shape          // initial cross-section data
[cross-section assect_num]  1         "block"            // "block" is defined below
End of file

<-------------------------------------------------------------------
			"block" is defined in this way:



			if (code != Cold) && (code != Poli) && (code != Composed) && (code != Juan)
			{
					"block"=
					   10           double   // a vector of sizes see shapes.doc
			}


			else if(code == Cold)
			{
			   	"block"=
						  1          char                // =1 open; =2 closed
				        1          long                // number of sides, nsides
					nsides          SR3_sh_side         // the sides					     
			}



			else if(code == Juan)
			{
			   	"block"=
						  1          double                // A
						  1          double                // A2
						  1          double                // A3
						  1          double                // Jt
						  1          double                // J2
						  1          double                // J3
						  1          double                // it
						  1          double                // i2
						  1          double                // i3
						  1          double                // W2
						  1          double                // W3
						  1          double                // Wpl2
						  1          double                // Wpl3
						  1          double                // U
						  1          double                // Cm
			}



			else if(code == Composed) 
			{
                              1          unsigned long         // the number "nscs" of simple cross-sections
[simple cross section 1]      1          double                // X of this simple shape   
[simple cross section 1]      1          double                // Y of this simple shape   
[simple cross section 1]      1          double                // angle of this simple shape   
[simple cross section 1]      1          SR3_Shape             // initial cross-section data
[simple cross section 1]     10          double                // parameters of this simple-cross section
[simple cross section 2]      1          double                // X of this simple shape   
[simple cross section 2]      1          double                // Y of this simple shape   
[simple cross section 2]      1          double                // angle of this simple shape   
[simple cross section 2]      1          SR3_Shape             // initial cross-section data
[simple cross section 2]     10          double                // parameters of this simple-cross section
....................
[simple cross section nscs]   1          double                // X of this simple shape   
[simple cross section nscs]   1          double                // Y of this simple shape   
[simple cross section nscs]   1          double                // angle of this simple shape   
[simple cross section nscs]   1          SR3_Shape             // initial cross-section data
[simple cross section nscs]  10          double                // parameters of this simple-cross section
			}



			else if(code == Poli)
			{
                              1          unsigned long         // the number "npol" of polygons
                              1          double                // the torsional inertia Jt
[polygon 1]                   1          unsigned char         // 0 is a hole; 1 is not a hole
[polygon 1]                   1          unsigned long         // npoint - number of point of this poly (a tria has 3 points)
[polygon1 point 1]            1          double                // X
[polygon1 point 1]            1          double                // Y
[polygon1 point 1]            1          double                // X
[polygon1 point 1]            1          double                // Y
...................
[polygon2 point npoint]       1          double                // X
[polygon2 point npoint]       1          double                // Y
[polygon 2]                   1          unsigned char         // 0 is a hole; 1 is not a hole
[polygon 2]                   1          unsigned long         // npoint - number of point of this poly (a tria has 3 points)
[polygon2 point 1]            1          double                // X
[polygon2 point 1]            1          double                // Y
[polygon2 point 1]            1          double                // X
[polygon2 point 1]            1          double                // Y
...................
[polygon2 point npoint]       1          double                // X
[polygon2 point npoint]       1          double                // Y
...........................................
...........................................
[polygon npol]                1          unsigned char         // 0 is a hole; 1 is not a hole
[polygon npol]                1          unsigned long         // npoint - number of point of this poly (a tria has 3 points)
[polygon npol point 1]        1          double                // X
[polygon npol point 1]        1          double                // Y
[polygon npol point 1]        1          double                // X
[polygon npol point 1]        1          double                // Y
...................
[polygon npol point npoint]   1          double                // X
[polygon npol point npoint]   1          double                // Y
			}
-------------------------------------------------------------------------------->
*/











/*


                    EXAMPLE to write .SR3 file







savfile:        pointer to the file to write into (blank binary file)
pstruc:         pointer to your data structure embedding fem model data

int sr3_write(FILE *savfile, YOUR_FORMAT_STRUCTURE *pstruc)
{
   SR3_Backup_buffer buffer;
   SR3_Backup_buffer1 buffer1;
   register unsigned long i;
   register unsigned long j, k;
   short version;

	strcpy(buffer.job, "");
	strcpy(buffer.program, "Sargon");
	buffer.version = 100;
	buffer.post = struc.post_st; // post processing availability
   if(!fwrite(&buffer, sizeof(SR3_Backup_buffer), 1, savfile))
   {
      fclose(savfile);
      return(1);
   }
	buffer1.asmat_num = struc.GetNumMaterials();
	buffer1.asprop_num = struc.GetNumPlmprops();
	buffer1.assect_num = struc.GetNumShapes();
	buffer1.beam_num = struc.GetBeamNum();
	buffer1.dummy1 = buffer1.dummy2 = buffer1.dummy3 = buffer1.dummy4 = 0;
	buffer1.dummy5 = buffer1.dummy6 = 0;
	buffer1.f_conv = 1.;
	buffer1.l_conv = 1.;
	buffer1.t_conv = 1.;
	buffer1.temp_conv = 1.;
	buffer1.temp_conv2 = 0.;
	buffer1.group_num = struc.GetGroupNum();
	buffer1.membr_num = struc.GetMembrNum();
	buffer1.ncombi = struc.GetNcombi();
	buffer1.nlcase = struc.GetNlcase();
	buffer1.node_num = struc.GetNodeNum();
	buffer1.plate_num = struc.GetPlateNum();
	buffer1.solid_num = struc.GetSolidNum();
	buffer1.spring_num = struc.GetSpringNum();
	buffer1.truss_num = struc.GetTrussNum();
   if(!fwrite(&buffer1, sizeof(SR3_Backup_buffer1), 1, savfile))
   {
      fclose(savfile);
      return(1);
   }
   if(buffer1.node_num)
   {
		SR3_R3node *nodes;
      if((nodes = (SR3_R3node *)malloc(buffer1.node_num * sizeof(SR3_R3node))) == NULL)
      {
         fclose(savfile);
         return(1);
      }
		// assignes data to nodes vect
		// ..................... 
		// and then
      if(fwrite(nodes, sizeof(SR3_R3node), buffer1.node_num, savfile) != buffer1.node_num)
      {
         fclose(savfile);
         return(1);
      }
		free(nodes);
   }
   if(buffer1.beam_num)
   {
		SR3_Beam *beams;
      if((beams = (SR3_Beam *)malloc(buffer1.beam_num * sizeof(SR3_Beam))) == NULL)
      {
         fclose(savfile);
         return(1);
      }
		// assignes data to beams vect
		// ..................... 
		// and then
      if(fwrite(beams, sizeof(SR3_Beam), buffer1.beam_num, savfile) != buffer1.beam_num)
      {
         fclose(savfile);
         return(1);
      }
		free(beams);
   }
   if(buffer1.truss_num)
   {
		SR3_Truss *trusses;
      if((trusses = (SR3_Truss *)malloc(buffer1.truss_num * sizeof(SR3_Truss))) == NULL)
      {
         fclose(savfile);
         return(1);
      }
		// assignes data to trusses vect
		// ..................... 
		// and then
      if(fwrite(trusses, sizeof(SR3_Truss), buffer1.truss_num, savfile) != buffer1.truss_num)
      {
         fclose(savfile);
         return(1);
      }
		free(trusses);
   }
   if(buffer1.plate_num)
   {
		SR3_Plate *plates;
      if((plates = (SR3_Plate *)malloc(buffer1.plate_num * sizeof(SR3_Plate))) == NULL)
      {
         fclose(savfile);
         return(1);
      }
		// assignes data to plates vect
		// ..................... 
		// and then
      if(fwrite(plates, sizeof(SR3_Plate), buffer1.plate_num, savfile) != buffer1.plate_num)
      {
         fclose(savfile);
         return(1);
      }
		free(plates);
   }
   if(buffer1.membr_num)
   {
		SR3_Membr *membrs;
      if((membrs = (SR3_Membr *)malloc(buffer1.membr_num * sizeof(SR3_Membr))) == NULL)
      {
         fclose(savfile);
         return(1);
      }
		// assignes data to membrs vect
		// ..................... 
		// and then
      if(fwrite(membrs, sizeof(SR3_Membr), buffer1.membr_num, savfile) != buffer1.membr_num)
      {
         fclose(savfile);
         return(1);
      }
		free(membrs);
   }
   if(buffer1.solid_num)
   {
		SR3_Solid *solids;
      if((solids = (SR3_Solid *)malloc(buffer1.solid_num * sizeof(SR3_Solid))) == NULL)
      {
         fclose(savfile);
         return(1);
      }
		// assignes data to solids vect
		// ..................... 
		// and then
      if(fwrite(solids, sizeof(SR3_Solid), buffer1.solid_num, savfile) != buffer1.solid_num)
      {
         fclose(savfile);
         return(1);
      }
		free(solids);
   }
   if(buffer1.spring_num)
   {
		SR3_Spring *springs;
      if((springs = (SR3_Spring *)malloc(buffer1.spring_num * sizeof(SR3_Spring))) == NULL)
      {
         fclose(savfile);
         return(1);
      }
		// assignes data to springs vect
		// ..................... 
		// and then
      if(fwrite(springs, sizeof(SR3_Spring), buffer1.spring_num, savfile) != buffer1.spring_num)
      {
         fclose(savfile);
         return(1);
      }
		free(springs);
   }
   if(buffer1.group_num)
   {
		SR3_Group *groups;
      if((groups = (SR3_Group *)malloc(buffer1.group_num * sizeof(SR3_Group))) == NULL)
      {
         fclose(savfile);
         return(1);
      }
		// assignes data to groups vect
		// ..................... 
		// and then
      if(fwrite(groups, sizeof(SR3_Group), buffer1.group_num, savfile) != buffer1.group_num)
      {
         fclose(savfile);
         return(1);
      }
		free(groups);
   }
   if(buffer1.nlcase)
   {
		SR3_Lc_header *lcases;
      if((lcases = (SR3_Lc_header *)malloc(buffer1.nlcase * sizeof(SR3_Lc_header))) == NULL)
      {
         fclose(savfile);
         return(1);
      }
		for(unsigned long i=0; i < buffer1.nlcase; i++)
		{
			// fill lcases[i]
		}
		if(fwrite(lcases, sizeof(SR3_Lc_header), buffer1.nlcase, savfile) != buffer1.nlcase)
      {
         fclose(savfile);
         return(1);
      }
		for(unsigned long i=0; i < buffer1.nlcase; i++)
		{
			if(lcases[i].gforces_num)
			{
				SR3_Gforce *gforces;
				if((gforces = (SR3_Gforce *) malloc(lcases[i].gforces_num * sizeof(SR3_Gforce))) == NULL)
				{
					fclose(savfile);
					return(1);
				}
				// fill gforces vect
				// .............
				// and then
				if(fwrite(gforces, sizeof(SR3_Gforce), lcases[i].gforces_num, savfile) != lcases[i].gforces_num)
				{
					fclose(savfile);
					return(1);
				}
				free(gforces);
			}
			//
			if(lcases[i].uloads_num)
			{
				SR3_Uload *uloads;
				if((uloads = (SR3_Uload *) malloc(lcases[i].uloads_num * sizeof(SR3_Uload))) == NULL)
				{
					fclose(savfile);
					return(1);
				}
				// fill uloads vect
				// .............
				// and then
				if(fwrite(uloads, sizeof(SR3_Uload), lcases[i].uloads_num, savfile) != lcases[i].uloads_num)
				{
					fclose(savfile);
					return(1);
				}
				free(uloads);
			}
			//
			if(lcases[i].cfloads_num)
			{
				SR3_Cfload *cfloads;
				if((cfloads = (SR3_Cfload *) malloc(lcases[i].cfloads_num * sizeof(SR3_Cfload))) == NULL)
				{
					fclose(savfile);
					return(1);
				}
				// fill cfloads vect
				// .............
				// and then
				if(fwrite(cfloads, sizeof(SR3_Cfload), lcases[i].cfloads_num, savfile) != lcases[i].cfloads_num)
				{
					fclose(savfile);
					return(1);
				}
				free(cfloads);
			}
			//
			if(lcases[i].lloads_num)
			{
				SR3_Lload *lloads;
				if((lloads = (SR3_Lload *) malloc(lcases[i].lloads_num * sizeof(SR3_Lload))) == NULL)
				{
					fclose(savfile);
					return(1);
				}
				// fill lloads vect
				// .............
				// and then
				if(fwrite(lloads, sizeof(SR3_Lload), lcases[i].lloads_num, savfile) != lcases[i].lloads_num)
				{
					fclose(savfile);
					return(1);
				}
				free(lloads);
			}
			//
			if(lcases[i].glloads_num)
			{
				SR3_Glload *glloads;
				if((glloads = (SR3_Glload *) malloc(lcases[i].glloads_num * sizeof(SR3_Glload))) == NULL)
				{
					fclose(savfile);
					return(1);
				}
				// fill glloads vect
				// .............
				// and then
				if(fwrite(glloads, sizeof(SR3_Glload), lcases[i].glloads_num, savfile) != lcases[i].glloads_num)
				{
					fclose(savfile);
					return(1);
				}
				free(glloads);
			}
			//
			if(lcases[i].tloads_num)
			{
				SR3_Tload *tloads;
				if((tloads = (SR3_Tload *) malloc(lcases[i].tloads_num * sizeof(SR3_Tload))) == NULL)
				{
					fclose(savfile);
					return(1);
				}
				// fill tloads vect
				// .............
				// and then
				if(fwrite(tloads, sizeof(SR3_Tload), lcases[i].tloads_num, savfile) != lcases[i].tloads_num)
				{
					fclose(savfile);
					return(1);
				}
				free(tloads);
			}
		}
		free(lcases);
   }
	if(buffer1.ncombi)
	{
		SR3_Combi_Header chead;
		double *mult = new double[buffer1.nlcase];
		for(unsigned long i=0; i < buffer1.ncombi; i++)
		{
			// fills chead for this combi
			// .........................
			// and then
			if(fwrite(&chead, sizeof(SR3_Combi_Header), 1, savfile) != 1)
			{
				fclose(savfile);
				return (1);
			}
			// fills mult for this combi
			// .........................
			// and then
			if(fwrite(mult, sizeof(double), buffer1.nlcase, savfile) != buffer1.nlcase)
			{
				fclose(savfile);
				return (1);
			}
			if(chead.nitem)
			{
				double *item_mult = new double[chead.nitem]
				unsigned long *item_num = new unsigned long[chead.nitem]
			   // fills item_mult and item_num for this combi 
				// ......................
				// and then writes
				if(fwrite(item_mult, sizeof(double), chead.nitem, savfile) != chead.nitem)
				{
					fclose(savfile);
					return (1);
				}
				if(fwrite(item_num, sizeof(unsigned long), chead.nitem, savfile) != chead.nitem)
				{
					fclose(savfile);
					return (1);
				}
			}
			delete [] item_mult;
			delete [] item_num;
		}
		delete []mult;
	}
   if(buffer1.asmat_num)
   {
		SR3_Material *mat_prop;
      if((mat_prop = (SR3_Material *)malloc(buffer1.asmat_num * sizeof(SR3_Material))) == NULL)
      {
         fclose(savfile);
         return(1);
      }
		// fills mat_prop vector
		// .........................
		// and then
      if(fwrite(mat_prop, sizeof(SR3_Material), buffer1.asmat_num, savfile) != buffer1.asmat_num)
      {
         fclose(savfile);
         return(1);
      }
		free(mat_prop);
   }
   if(buffer1.asprop_num)
   {
		SR3_Plmprop *plmprops;
      if((plmprops = (SR3_Plmprop *)malloc(buffer1.asprop_num * sizeof(SR3_Plmprop))) == NULL)
      {
         fclose(savfile);
         return(1);
      }
		// fills plmprops vector
		// .........................
		// and then
      if(fwrite(plmprops, sizeof(SR3_Plmprop), buffer1.asprop_num, savfile) != buffer1.asprop_num)
      {
         fclose(savfile);
         return(1);
      }
		free(plmprops);
   }
   if(buffer1.assect_num)
   {
		Shape *p;
		ShapeColl *pcoll = struc.GetPShpColl();
		for(unsigned long i=0; i < buffer1.assect_num; i++)
		{
			SR3_Shape shap;
			// fills shap for this cross-section shape
			// .........................
			// and then
	      if(fwrite(&shap, sizeof(SR3_Shape), 1, savfile) != 1)
	      {
	         fclose(savfile);
	         return(1);
	      }
			if((shap.code != SR3_Juan) && (shap.code != SR3_Cold) && (shap.code != SR3_Composed) &&
				(shap.code != SR3_Poli) && (shap.code != SR3_Ipn) && (shap.code != SR3_Tsm))
			{
				double param[10];	
				// fills param vector for this cross-section shape
				// .........................
				// and then
		      if(fwrite(&param, sizeof(double), 10, savfile) != 10)
		      {
		         fclose(savfile);
		         return(1);
		      }
			}
			else if(shap.code == SR3_Juan)
			{
				double param[15];
				// fills param vector for this cross-section shape of kind "Juan"
				// .........................
				// and then
		      if(fwrite(&param, sizeof(double), 15, savfile) != 15)
		      {
		         fclose(savfile);
		         return(1);
		      }
			}
			else if(shap.code == SR3_Cold)
			{
				char end = get end code for this cold cross-section
		      if(fwrite(&end, sizeof(char), 1, savfile) != 1)
		      {
		         fclose(savfile);
		         return(1);
		      }
				long nside = get the number of sides
		      if(fwrite(&nside, sizeof(long), 1, savfile) != 1)
		      {
		         fclose(savfile);
		         return(1);
		      }
				SR3_sh_side *pside = new SR3_sh_side[nside];
				// fills pside vector for this cold cross-section shape
				// .........................
				// and then
		      if(fwrite(pside, sizeof(SR3_sh_side), nside, savfile) != nside)
		      {
		         fclose(savfile);
		         return(1);
		      }
				delete []pside;
			}
			else if(shap.code == SR3_Poli)
			{
				unsigned long npoli = get the number of polygon for this cross-section
		      if(fwrite(&npoli, sizeof(long), 1, savfile) != 1)
		      {
		         fclose(savfile);
		         return(1);
		      }
				double Jt = get the torsional inertia
		      if(fwrite(&Jt, sizeof(double), 1, savfile) != 1)
		      {
		         fclose(savfile);
		         return(1);
		      }
				for(long j=0; j < npoli; j++)
				{
					unsigned char hole = get the hole code for this polygon
					if(fwrite(&hole, sizeof(unsigned char), 1, savfile) != 1)
					{
						fclose(savfile);
						return(1);
					}
					unsigned long npoint = get the number of point for this polygon
					if(fwrite(&npoint, sizeof(unsigned long), 1, savfile) != 1)
					{
						fclose(savfile);
						return(1);
					}
					double *coords = new double[npoint * 2];
					// fills coords vector for this polygon of this cross-section shape
					// .........................
					// and then
					if(fwrite(coords, sizeof(double), 2 * npoint, savfile) != 2*npoint)
					{
						fclose(savfile);
						return (1);
					}
					delete []coords;
				}
			}
			else if(shap.code == SR3_Composed)
			{
				unsigned long nscs = get the number of simple cross-sections
				double data[3];
				if(fwrite(&nscs, sizeof(unsigned long), 1, savfile) != 1)
				{
					fclose(savfile);
					return(1);
				}
				// loop on simple cross-section composing this SR3_Composed cross-section
				for(long j=0; j < nscs; j++)
				{
					data[0] = get X of this simple cross-section
					data[1] = get Y of this simple cross-section
					data[2] = get Angle of this simple cross-section
					if(fwrite(data, sizeof(double), 3, savfile) != 3)
					{
						fclose(savfile);
						return(1);
					}
					SR3_Shape sh;
					sh.code = get code of this simple cross-section composing this cross-section
					// get name, e.g.: strcpy_s(sh.name, 20, q->GetShape(j+1)->GetLabel());
					if(fwrite(&sh, sizeof(SR3_Shape), 1, savfile) != 1)
					{
						fclose(savfile);
						return(1);
					}
					double param[10];
					// fills the param vector for this simple cross shape
					// ................
					// and then
					if(fwrite(param, sizeof(double), 10, savfile) != 10)
					{
						fclose(savfile);
						return(1);
					}
				}
			}
		}
   }
   fclose(savfile);
   return(0);
}


*/


#endif
